
 
const nsICollection = Components.interfaces.nsICollection;
const nsIConsoleService = Components.interfaces.nsIConsoleService;
const nsISupportsCString = Components.interfaces.nsISupportsCString;
const nsISupportsString = Components.interfaces.nsISupportsString;
const nsIPrefService = Components.interfaces.nsIPrefService;
 
const XPCOM_CONTRACTID = '@scenari.com/scSession;1';
const XPCOM_CID = Components.ID('{A7A3079A-7FD9-4E6F-A278-45A02C9A3333}');
const XPCOM_IID = Components.interfaces.scISession;



function scSession() {
	this.fBaseStorageUrl 	= "";
	this.fStorageFile 		= "";
	this.fUid 				= "";
	this.fName 				= "";
	this.fModuleUrl 		= "";
}


scSession.prototype = {
  /**
	* Initialise la session
	**/
	init : function(pUid, pModuleUrl, pStoragePath, pName) {
		try{
			this.fName = pName;
			this.fUid = pUid;
			this.fModuleUrl = pModuleUrl;
			this.setBaseStoragePath(pStoragePath);
		}catch(e){
			debug("scSession > init :: " + e);
		}
	},

   /**
	 * Supprime toutes les productions de cette session (le repertoire de donnees)
	 **/
	remove : function() {
		try{
			if(this.fStorageFile.exists()){
				var vStorageFolder = this.fStorageFile.parent;
				this.fStorageFile.remove(true);
			}
			return 0;
		}catch(e){
			debug("scSession > remove :: " + e);
			return Components.interfaces.scISession.SC_ERR_NOT_DEFINED;
		}
	},
   /**
	 * Retourne l'identifiant unique de cette session
	 **/
	getUid : function() {
		try{
			return this.fUid;
		}catch(e){
			debug("scSession > getUid :: " + e);
		}
	},
   /**
	 * Retourne le nom de cette session
	 **/
	getName : function() {
		try{
			return this.fName;
		}catch(e){
			debug("scSession > getName :: " + e);
		}
	},
   /**
	 * Retourne l'URL du module de cette session
	 **/
	getModuleUrl : function() {
		try{
			return this.fModuleUrl;
		}catch(e){
			debug("scSession > getModuleUrl :: " + e);
		}
	},
   /**
	 * (re)Affecte l'URL de stockage des productions de cette session
	 *   Note : le changement de place des donnees a deja ete effectue par le sessionsMgr
	 **/
	setBaseStoragePath : function(pUrl) {
		try{
			this.fBaseStorageUrl = pUrl;
			var vCurrentFile = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
			vCurrentFile.initWithPath(this.fBaseStorageUrl);
			vCurrentFile.append(this.fUid);
			this.fStorageFile = vCurrentFile;
		}catch(e){
			debug("scSession > setBaseStoragePath :: " + e);
		}
	},
   /**
	 * Enregistre la valeur d'un champs
	 **/
	setData : function(pId, pValue) {
		var vOutputStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance( Components.interfaces.nsIFileOutputStream );
		try{
			if(!this.fStorageFile.exists()) this.fStorageFile.create(Components.interfaces.nsIFile.DIRECTORY_TYPE, 0755);
			if(!this.fStorageFile.diskSpaceAvailable>pValue.length*8) return Components.interfaces.scISession.SC_ERR_NOT_ENOUGH_SPACE;
			if(this.fStorageFile.exists() && !this.fStorageFile.isWritable()) return Components.interfaces.scISession.SC_ERR_NO_WRITABLE;
			var vCurrentDataFile = this.fStorageFile.clone();
			vCurrentDataFile.append(pId + '.dat');
			if (!vCurrentDataFile.exists()) vCurrentDataFile.create(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 0644);
			vOutputStream.init(vCurrentDataFile, 0x04 | 0x08 | 0x20, 0644, 0);
			vOutputStream.write(pValue, pValue.length);
			return 0;
		}catch(e){
			debug("scSession > setData :: " + e);
			return Components.interfaces.scISession.SC_ERR_NOT_DEFINED;
		}finally{
			vOutputStream.close();
		}
	},
   /**
	 * Enregistre la valeur d'un champs
	 **/
	getData : function(pId) {
		try{
			var vCurrentDataFile = this.fStorageFile.clone();
			vCurrentDataFile.append(pId + '.dat');
			if (vCurrentDataFile.exists()){
				var vIS = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(Components.interfaces.nsIFileInputStream);
				vIS.init(vCurrentDataFile,0x01, 00004, null);
				var vSIS = Components.classes["@mozilla.org/scriptableinputstream;1"].createInstance(Components.interfaces.nsIScriptableInputStream);
				vSIS.init(vIS);
				return vSIS.read(vSIS.available());
			}else{
				return "";
			}
		}catch(e){
			debug("scSession > getData :: " + e);
		}
	},
	 /**
	 * Enregistre un fichier
	 **/
	setFile : function(pId, pUrl) {
		try{
			debug("setFile non implementee");

			return 0;
		}catch(e){
			debug("scSession > setFile :: " + e);
		}
	},
   /**
	 * retourne un fichier
	 **/
	getFile : function(pId) {
		try{
			debug("getFile non implementee");

		}catch(e){
			debug("scSession > getFile :: " + e);
		}
	},
	QueryInterface: function(pIId) {
        if (!pIId.equals(nsISupports) &&
            !pIId.equals(XPCOM_CID))
            throw Components.results.NS_ERROR_NO_INTERFACE;
        return this;
    }
}



function debug(pText){
	var vConsoleService = Components.classes["@mozilla.org/consoleservice;1"].getService(nsIConsoleService);
	vConsoleService.logStringMessage(pText);
}

function listProperties (pObj){
	var vRes = "";
	var vProp = null;
	for (vProp in pObj) {
		try {
			vRes += vProp +'(' + pObj[vProp]+ ')'+ "\n";
		} catch(e){
			vRes += vProp +' -> toString in error\n';
		}
	}
	return vRes;
}



var scModule = {
    registerSelf: function(compMgr, fileSpec, location, type) {
        compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
        compMgr.registerFactoryLocation(XPCOM_CID, 
                                        "scSession JS component", 
                                        XPCOM_CONTRACTID, 
                                        fileSpec, 
                                        location,
                                        type);
                            	
    },

    getClassObject: function(compMgr, cid, iid) {
        if (!cid.equals(XPCOM_CID))
            throw Components.results.NS_ERROR_NO_INTERFACE;

        if (!iid.equals(Components.interfaces.nsIFactory))
            throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

        return scSessionFactory;
    },
    
    unregisterSelf : function mod_unreg(compMgr, location, type) {
  
  	},

    canUnload: function(compMgr) { return true; }
};


var scSessionFactory = {
    createInstance: function(outer, iid) {
    	try{
	        if (outer != null)
	            throw Components.results.NS_ERROR_NO_AGGREGATION;
	        if (!iid.equals(XPCOM_IID) &&
	            !iid.equals(Components.interfaces.nsISupports))
	            throw Components.results.NS_ERROR_INVALID_ARG;
	      }catch(e){
	      	debug("scSession > createInstance :: " + e);
	      }finally{
	      	return new scSession();
	      }
    }
}


function NSGetModule(comMgr, fileSpec) { return scModule; }

